@@ -3,7 +3,7 @@
 # scantwo.R
 #
 # copyright (c) 2001-2015, Karl W Broman and Hao Wu
-# last modified Aug, 2015
+# last modified Oct, 2015
 # first written Nov, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -23,7 +23,6 @@
 #
 # Part of the R/qtl package
 # Contains: scantwo, scantwo.perm, scantwo.perm.engine
-#           revisescantwodf
 #
 ######################################################################
 
@@ -92,8 +91,7 @@ scantwo <-
         for(i in which(chrtype=="X")) class(cross$geno[[i]]) <- "A"
 
     # X-chr-specific perms (actually A:A, A:X, and X:X specific)
-    if(perm.Xsp && n.perm > 0) {
-        if(all(chrtype=="X") || all(chrtype=="A")) break # no need for x-chr-specific perms
+    if(perm.Xsp && n.perm > 0 && !(all(chrtype=="X") || all(chrtype=="A"))) { # no need for x-chr-specific perms
         if(length(chr1) != length(chr2) || any(chr1 != chr2))
             stop("With perm.Xsp=TRUE, chr can't be a list")
 
@@ -207,10 +205,6 @@ scantwo <-
         pheno.col <- 1
     }
 
-    dfAAf <- dfAAa <- dfAA1 <- -1
-    dfAXf <- dfAXa <- dfAX1 <- -1
-    dfXXf <- dfXXa <- dfXX1 <- -1
-
     origcross <- cross
 
     fullmap <- pull.map(cross)
@@ -314,7 +308,7 @@ scantwo <-
 
             temp <- scantwo(subset(cross, ind=upat[[i]]), chr=chr, pheno.col=batches[[i]], model=model,
                             method=method, addcovar=tempac, intcovar=tempic,
-                            weights=weights, use=use, incl.markers=incl.markers, clean.output=clean.output,
+                            weights=weights[upat[[i]]], use=use, incl.markers=incl.markers, clean.output=clean.output,
                             clean.nmar=clean.nmar, clean.distance=clean.distance,
                             maxit=maxit, tol=tol, verbose=verbose, n.perm=n.perm,
                             perm.strata=perm.strata[upat[[i]]], assumeCondIndep=assumeCondIndep,
@@ -479,9 +473,6 @@ scantwo <-
                     addcovar=addcovar, intcovar=intcovar, weights=weights,
                     use=use, maxit=maxit, tol=tol, verbose=FALSE)
     out.scanone <- temp[,-(1:2),drop=FALSE]
-    dfAA1 <- attr(temp, "df")["A"]
-    if(any(chrtype=="X"))
-        dfAX1 <- dfXX1 <- attr(temp, "df")["X"]
     if(verbose) cat(" --Running scantwo\n")
 
     if(method=="mr" || method=="mr-imp" || method=="mr-argmax") { # marker regression
@@ -673,21 +664,6 @@ scantwo <-
                 n.col2drop <- 0
             }
 
-            # get degrees of freedom
-            if(dfAAf < 0 && chrtype[i]=="A" && chrtype[j]=="A") {
-                dfAAf <- (n.gen[i]*n.gen[j]-1)*(n.ic+1)
-                dfAAa <- (n.gen[i]+n.gen[j]-2)*(n.ic+1)
-            }
-            else if(dfAXf < 0 && ((chrtype[i]=="A" && chrtype[j]=="X") ||
-                                  (chrtype[i]=="X" && chrtype[j]=="A"))) {
-                dfAXf <- (n.gen[i]*n.gen[j]-1)*(n.icX+1)
-                dfAXa <- (n.gen[i]+n.gen[j]-2)*(n.icX+1)
-            }
-            if(dfXXf < 0 && chrtype[i]=="X" && chrtype[j]=="X") {
-                dfXXf <- (n.gen[i]*n.gen[j]-1-n.col2drop)*(n.icX+1)
-                dfXXa <- (n.gen[i]+n.gen[j]-2-n.col2drop.addmodel)*(n.icX+1)
-            }
-
             # print the current working pair
             if(verbose) cat(" (", names(cross$geno)[i], ",",
                             names(cross$geno)[j],")\n",sep="")
@@ -1460,11 +1436,6 @@ scantwo <-
             }
         }
 
-        dfXXf <- dfXXf - parX0 + 1
-        dfXXa <- dfXXa - parX0 + 1
-
-        dfAXf <- dfAXf - parX0 + 1
-        dfAXa <- dfAXa - parX0 + 1
     }
 
     #  if(any(is.na(results)) && n.perm > -2)
@@ -1496,16 +1467,6 @@ scantwo <-
     attr(out, "fullmap") <- fullmap
     class(out) <- "scantwo"
 
-    df <- NULL
-    if(any(chrtype=="A"))
-        df <- rbind(df, "AA"=c("full"=dfAAf, "add"=dfAAa, "one"=dfAA1))
-    if(any(chrtype=="A") && any(chrtype=="X"))
-        df <- rbind(df, "AX"=c("full"=dfAXf, "add"=dfAXa, "one"=dfAX1))
-    if(any(chrtype=="X"))
-        df <- rbind(df, "XX"=c("full"=dfXXf, "add"=dfXXa, "one"=dfXX1))
-    colnames(df) <- c("full","add","one")
-    attr(out, "df") <- df
-
     if(clean.output) # remove NA, 0 out positions between markers
         out <- clean(out, clean.nmar, clean.distance)
 
@@ -1658,9 +1619,6 @@ scantwo.perm.engine <-
                 ## find the maximum LOD on each permutation
                 if(is.null(perm.result)) {
                     perm.result <- lapply(subrousummaryscantwo(tem,for.perm=TRUE), as.matrix)
-
-                    if("df" %in% names(attributes(tem)))
-                        attr(perm.result, "df") <- revisescantwodf(attr(tem, "df"))
                 }
                 else {
                     tem <- lapply(subrousummaryscantwo(tem,for.perm=TRUE), as.matrix)
@@ -1765,8 +1723,6 @@ scantwo.perm.engine <-
             # maxima
             for(j in 1:6) perm.result[[j]][i,] <- temp[[j]]
 
-            if("df" %in% names(attributes(tem)))
-                attr(perm.result, "df") <- revisescantwodf(attr(tem, "df"))
         }
     }
 
@@ -1782,17 +1738,4 @@ scantwo.perm.engine <-
 }
 
 
-revisescantwodf <-
-    function(df)
-{
-    if(is.null(df)) return(NULL)
-    full <- df[,"full"]
-    add <- df[,"add"]
-    one <- df[,"one"]
-    out <- cbind(full=full, fv1=full-one, int=full-add,
-                 add=add, av1=add-one)
-    rownames(out) <- rownames(df)
-    out
-}
-
 # end of scantwo.R
