@@ -2,8 +2,8 @@
 #
 # summary.scanone.R
 #
-# copyright (c) 2001-2012, Karl W Broman
-# last modified Nov, 2012
+# copyright (c) 2001-2018, Karl W Broman
+# last modified Feb, 2018
 # first written Sep, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -32,7 +32,7 @@
 ##################################################################
 summary.scanone <-
     function(object, threshold, format=c("onepheno", "allpheno", "allpeaks", "tabByCol", "tabByChr"),
-             perms, alpha, lodcolumn=1, pvalues=FALSE, df=FALSE,
+             perms, alpha, lodcolumn=1, pvalues=FALSE,
              ci.function=c("lodint", "bayesint"), ...)
 {
     if(!any(class(object) == "scanone"))
@@ -138,15 +138,6 @@ summary.scanone <-
         warning("Can show p-values only if perms are provided.")
         pvalues <- FALSE
     }
-
-    if(!missing(perms) && "df" %in% names(attributes(object)) &&
-       "df" %in% names(attributes(perms))) {
-        df.o <- attr(object, "df")
-        df.p <- attr(perms, "df")
-        if(length(df.o) != length(df.p) ||
-           any(df.o != df.p))
-            warning("Degrees of freedom in input object and perms do not match.")
-    }
     # end of check of input
 
     # chromosome IDs as a character string
@@ -510,10 +501,6 @@ summary.scanone <-
         attr(result, "tab") <- format
     }
 
-    if(df && "df" %in% names(attributes(object)))
-        attr(result, "df") <- attr(object, "df")
-    if(!df) attr(result, "df") <- NULL
-
     if(format=="allpeaks") rownames(result) <- as.character(result$chr)
 
     if(format=="tabByCol" || format=="tabByChr")
@@ -534,20 +521,6 @@ print.summary.scanone <-
         return(invisible(NULL))
     }
 
-    if("df" %in% names(attributes(x))) {
-        df <- attr(x, "df")
-        cat("Degrees of freedom: ")
-        if(is.matrix(df)) { # for 2-part model
-            cat("\n")
-            rownames(df) <- paste("   ", rownames(df), ": ", sep="")
-            print(df)
-        }
-        else {
-            cat(paste(names(df), ":", df, sep="", collapse="  "), "\n")
-        }
-        cat("\n")
-    }
-
     flag <- FALSE
     if(is.null(tab)) {
         print.data.frame(x,digits=3)
@@ -581,15 +554,11 @@ print.summary.scanone <-
 
 # pull out maximum LOD peak, genome-wide
 max.scanone <-
-    function(object, chr, lodcolumn=1, df=FALSE, na.rm=TRUE, ...)
+    function(object, chr, lodcolumn=1, na.rm=TRUE, ...)
 {
     if(!any(class(object) == "scanone"))
         stop("Input must have class \"scanone\".")
 
-    if("df" %in% names(attributes(object)))
-        thedf <- attr(object, "df")
-    else thedf <- NULL
-
     if(lodcolumn < 1 || lodcolumn+2 > ncol(object))
         stop("Argument lodcolumn misspecified.")
 
@@ -601,9 +570,8 @@ max.scanone <-
     object <- object[wh,]
 
     object[,1] <- factor(as.character(unique(object[,1])))
-    attr(object, "df") <- thedf
 
-    summary.scanone(object,threshold=0,lodcolumn=lodcolumn, df=df)
+    summary.scanone(object,threshold=0,lodcolumn=lodcolumn)
 }
 
 ######################################################################
@@ -618,10 +586,6 @@ subset.scanone <-
     if(missing(chr) && missing(lodcolumn))
         stop("You must specify either chr or lodcolumn.")
 
-    if("df" %in% names(attributes(x)))
-        df <- attr(x, "df")
-    else df <- NULL
-
     y <- x
 
     if(!missing(chr)) {
@@ -652,7 +616,6 @@ subset.scanone <-
     if("model" %in% nam)
         attr(x, "model") <- attr(y,"model")
 
-    attr(x, "df") <- df
     x
 }
 
@@ -675,19 +638,6 @@ c.scanone <-
             stop("Input should have class \"scanone\".")
     }
 
-    df <- lapply(dots, attr, "df")
-    flag <- 0
-    for(i in 2:length(df)) {
-        if(length(df[[i]]) != length(df[[1]]) ||
-           any(df[[i]] != df[[1]]))
-            flag <- 1
-    }
-    if(flag) {
-        warning("Mismatch in degrees of freedom; may cause problems.")
-        df <- NULL
-    }
-    else df <- df[[1]]
-
     if(!missing(labels)) {
         if(length(labels)==1)
             labels <- rep(labels, length(dots))
@@ -733,7 +683,6 @@ c.scanone <-
                         as.data.frame(dots[[i]][,-(1:2),drop=FALSE], stringsAsFactors=TRUE))
 
     class(result) <- cl
-    attr(result, "df") <- df
     result
 }
 
@@ -761,7 +710,7 @@ grab.arg.names <-
 # scanone permutation test (from scanone with n.perm > 0)
 ######################################################################
 summary.scanoneperm <-
-    function(object, alpha=c(0.05, 0.10), df=FALSE, controlAcrossCol=FALSE, ...)
+    function(object, alpha=c(0.05, 0.10), controlAcrossCol=FALSE, ...)
 {
     if(!any(class(object) == "scanoneperm"))
         stop("Input should have class \"scanoneperm\".")
@@ -811,9 +760,6 @@ summary.scanoneperm <-
             quant[[k]] <- qu
         }
 
-        if(df && "df" %in% names(attributes(object)))
-            attr(quant,"df") <- attr(object, "df")
-
         attr(quant, "n.perm") <- c("A"=nrow(object$A), "X"=nrow(object$X))
         class(quant) <- "summary.scanoneperm"
     }
@@ -849,9 +795,6 @@ summary.scanoneperm <-
         }
         else rownames(quant) <- paste(100*alpha, "%", sep="")
 
-        if(df && "df" %in% names(attributes(object)))
-            attr(quant,"df") <- attr(object, "df")
-
         attr(quant, "n.perm") <- nrow(object)
         class(quant) <- "summary.scanoneperm"
     }
@@ -861,20 +804,6 @@ summary.scanoneperm <-
 print.summary.scanoneperm <-
     function(x, ...)
 {
-    if("df" %in% names(attributes(x))) {
-        df <- attr(x, "df")
-        cat("Degrees of freedom: ")
-        if(is.matrix(df)) { # for 2-part model
-            cat("\n")
-            rownames(df) <- paste("   ", rownames(df), ": ", sep="")
-            print(df)
-        }
-        else {
-            cat(paste(names(df), ":", df, sep="", collapse="  "), "\n")
-        }
-        cat("\n")
-    }
-
     n.perm <- attr(x, "n.perm")
     if(length(n.perm)==2) {
         cat("Autosome LOD thresholds (", n.perm[1], " permutations)\n", sep="")
@@ -909,19 +838,6 @@ rbind.scanoneperm <- c.scanoneperm <-
             stop("Input should have class \"scanoneperm\".")
     }
 
-    df <- lapply(dots, attr, "df")
-    flag <- 0
-    for(i in 2:length(df)) {
-        if(length(df[[i]]) != length(df[[1]]) ||
-           any(df[[i]] != df[[1]]))
-            flag <- 1
-    }
-    if(flag) {
-        warning("Mismatch in degrees of freedom; may cause problems.")
-        df <- NULL
-    }
-    else df <- df[[1]]
-
     if("xchr" %in% names(attributes(dots[[1]]))) {
         xchr <- lapply(dots, attr, "xchr")
         L <- lapply(dots, attr, "L")
@@ -958,7 +874,6 @@ rbind.scanoneperm <- c.scanoneperm <-
             result <- rbind(result, dots[[i]])
         class(result) <- "scanoneperm"
     }
-    attr(result, "df") <- df
     result
 }
 
@@ -976,19 +891,6 @@ cbind.scanoneperm <-
             stop("Input should have class \"scanoneperm\".")
     }
 
-    df <- lapply(dots, attr, "df")
-    flag <- 0
-    for(i in 2:length(df)) {
-        if(length(df[[i]]) != length(df[[1]]) ||
-           any(df[[i]] != df[[1]]))
-            flag <- 1
-    }
-    if(flag) {
-        warning("Mismatch in degrees of freedom; may cause problems.")
-        df <- NULL
-    }
-    else df <- df[[1]]
-
     if(!missing(labels)) {
         if(length(labels)==1)
             labels <- rep(labels, length(dots))
@@ -1083,7 +985,6 @@ cbind.scanoneperm <-
 
         class(result) <- "scanoneperm"
     }
-    attr(result, "df") <- df
     result
 }
 
@@ -1101,11 +1002,11 @@ subset.scanoneperm <-
             x <- lapply(x, as.matrix)
 
         if(missing(lodcolumn)) lodcolumn <- 1:ncol(x[[1]])
-        else if(!is.null(attr(try(x[[1]][,lodcolumn], silent=TRUE),"try-error")))
+        else if(!check_colindex(lodcolumn, x[[1]]))
             stop("lodcolumn misspecified.")
 
         if(missing(repl)) repl <- 1:nrow(x[[1]])
-        else if(!is.null(attr(try(x[[1]][repl,], silent=TRUE),"try-error")))
+        else if(!check_rowindex(repl, x[[1]]))
             stop("repl misspecified.")
 
         cl <- class(x)
@@ -1116,11 +1017,11 @@ subset.scanoneperm <-
         if(!is.matrix(x)) x <- as.matrix(x)
 
         if(missing(lodcolumn)) lodcolumn <- 1:ncol(x)
-        else if(!is.null(attr(try(x[,lodcolumn], silent=TRUE),"try-error")))
+        else if(!check_colindex(lodcolumn, x))
             stop("lodcolumn misspecified.")
 
         if(missing(repl)) repl <- 1:nrow(x)
-        else if(!is.null(attr(try(x[repl,], silent=TRUE),"try-error")))
+        else if(!check_rowindex(repl, x))
             stop("repl misspecified.")
 
         cl <- class(x)
@@ -1141,4 +1042,44 @@ subset.scanoneperm <-
     function(x, repl, lodcolumn)
     subset.scanoneperm(x, repl, lodcolumn)
 
+# function to check if column indices are okay
+check_colindex <-
+    function(index, mat)
+{
+    if(any(is.na(index))) return(FALSE)
+
+    n <- ncol(mat)
+
+    if(is.logical(index) && length(index) != n)
+        return(FALSE)
+    if(is.numeric(index)) {
+        if(any(index <= 0) && !all(index < 0)) return(FALSE) # don't mix pos and neg
+        if(all(index < 0) && any(index < -n)) return(FALSE) # out of bounds
+        if(all(index > 0) && any(index > n)) return(FALSE) # out of bounds
+    }
+    if(is.character(index) && !all(index %in% colnames(mat))) return(FALSE)
+
+    TRUE
+}
+
+# function to check if row indices are okay
+check_rowindex <-
+    function(index, mat)
+{
+    if(any(is.na(index))) return(FALSE)
+
+    n <- nrow(mat)
+
+    if(is.logical(index) && length(index) != n)
+        return(FALSE)
+    if(is.numeric(index)) {
+        if(any(index <= 0) && !all(index < 0)) return(FALSE) # don't mix pos and neg
+        if(all(index < 0) && any(index < -n)) return(FALSE) # out of bounds
+        if(all(index > 0) && any(index > n)) return(FALSE) # out of bounds
+    }
+    if(is.character(index) && !all(index %in% rownames(mat))) return(FALSE)
+
+    TRUE
+}
+
 # end of summary.scanone.R
